from dateutil.parser import parse
import json
from operator import itemgetter
import os

from django.core.cache import cache
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.views.decorators.http import require_http_methods
from django.shortcuts import redirect, render_to_response
from django.db.models import Max
from django.template import RequestContext
from django_oauth_client.decorators import oauth_token_required
from django_oauth_client.utils import get_client
from django.conf import settings

from models import ShortURL

from . import cb62

from u1to.utils import u1url

def dispatch(request, username, idx, extra):
    try:
        url = ShortURL.objects.get(user__username=username,
                                   index=cb62.decode(idx, username))
    except ShortURL.DoesNotExist:
        raise Http404
    if extra.startswith(url.extra):
        extra = extra[len(url.extra):]
    return HttpResponseRedirect(url.original_url + extra)


def _index(request):
    user = request.user
    client = get_client(user, settings.U1_SERVER)
    if not user.last_name:
        response, content = client.request(u1url('file_api_base', '/api/account/'))
        account_info = json.loads(content)
        user.last_name = account_info['nickname']
        user.save()

    cache_key = "files-%s" % user.username
    files = cache.get(cache_key)
    if files is None:
        response, content = client.request(
         u1url('file_api_base', '/api/file_storage/v1/public_files'))
        # content = file('fake.json').read()
        try:
            files = json.loads(content)
        except ValueError:
            # handle bad json
            raise

        cache.set(cache_key, files, 60)

    files.sort(key=itemgetter(u'when_changed'), reverse=True)

    pubfiles = dict((i.original_url, i)
                    for i in ShortURL.objects.filter(user=request.user))
    for f in files:
        path = f['resource_path'].lstrip('/')
        f['basename'] = os.path.basename(path)
        max_len = 80
        if len(path) > max_len:
            path = path[-(max_len-1):]
            f['trunc_path_prefix'] = u"\N{HORIZONTAL ELLIPSIS}"
        f['trunc_path'] = path
        # f['when_changed'] = parse(f['when_changed'])
        f['short_url'] = pubfiles.get(f['public_url'], None)

    return render_to_response(
        'index.html', {'files': files},
        context_instance=RequestContext(request))


@login_required
@oauth_token_required(settings.U1_SERVER)
@require_http_methods(['GET', 'POST'])
def index(request):
    if request.method == 'GET':
        return _index(request)
    verb = request.POST.get('verb')
    if verb == 'delete':
        return _delete(request)
    elif verb == 'create':
        return _create(request)
    else:
        r = HttpResponse()
        r.status_code = 405
        return r


def _delete(request):
    idx = request.POST.get('index')
    url = ShortURL.objects.get(user=request.user,
                               index=idx)
    url.delete()
    return redirect("index")

def _create(request):
    idx = ShortURL.objects.filter(user=request.user).aggregate(Max('index'))['index__max']
    if idx is None:
        idx = 1
    else:
        idx = idx + 1

    url = ShortURL.objects.create(user=request.user,
                                  original_url=request.POST['url'],
                                  index=idx,
                                  extra=request.POST['extra'])
    url.save()
    return redirect("index")
